# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Autopilot Functional Test Tool
# Copyright (C) 2012,2013,2014 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


"""Autopilot test result classes"""

from __future__ import absolute_import

import logging

from testtools import (
    ExtendedToOriginalDecorator,
    ExtendedToStreamDecorator,
    TestResultDecorator,
    TextTestResult,
    try_import,
)

from autopilot.utilities import _raise_on_unknown_kwargs


class StatusFormatTestResult(TextTestResult):
    """A TextTestResult that prints status messages to a stream."""

    def __init__(self, stream, failfast):
        super(StatusFormatTestResult, self).__init__(stream, failfast)
        self.stream = stream

    def startTest(self, test):
        self.stream.write(u'%s' % test.id())
        self.stream.write(u' ... ')
        self.stream.flush()
        super(StatusFormatTestResult, self).startTest(test)

    def stopTest(self, test):
        self.stream.write(u'\n')
        self.stream.flush()
        super(StatusFormatTestResult, self).stopTest(test)

    def stopTestRun(self):
        self.stream.write(u'-' * 80)
        self.stream.write(u'\n')
        self.stream.flush()
        super(StatusFormatTestResult, self).stopTestRun()

    def addExpectedFailure(self, test, err=None, details=None):
        self.stream.write(u'EXPECTED FAIL')
        super(StatusFormatTestResult, self).addExpectedFailure(
            test, err, details
        )

    def addError(self, test, err=None, details=None):
        self.stream.write(u'ERROR')
        super(StatusFormatTestResult, self).addError(
            test, err, details
        )

    def addFailure(self, test, err=None, details=None):
        self.stream.write(u'FAIL')
        super(StatusFormatTestResult, self).addFailure(
            test, err, details
        )

    def addSkip(self, test, reason, details=None):
        if not reason:
            reason_displayed = ''
        else:
            reason_displayed = ' (%s)' % reason
        self.stream.write(u'SKIP%s' % reason_displayed)
        super(StatusFormatTestResult, self).addSkip(
            test, reason, details
        )

    def addSuccess(self, test, details=None):
        self.stream.write(u'OK')
        super(StatusFormatTestResult, self).addSuccess(
            test, details
        )

    def addUnexpectedSuccess(self, test, details=None):
        self.stream.write(u'UNEXPECTED SUCCESS')
        super(StatusFormatTestResult, self).addUnexpectedSuccess(
            test, details
        )


class LoggedTestResultDecorator(TestResultDecorator):
    """A decorator that logs messages to python's logging system."""

    def _log(self, level, message):
        """Performs the actual message logging"""
        logging.getLogger().log(level, message)

    def _log_details(self, level, details):
        """Logs the relavent test details"""
        for detail in details:
            # Skip the test-log as it was logged while the test executed
            if detail == "test-log":
                continue
            text = "%s: {{{\n%s}}}" % (detail, details[detail].as_text())
            self._log(level, text)

    def addSuccess(self, test, details=None):
        self._log(logging.INFO, "OK: %s" % (test.id()))
        return super(LoggedTestResultDecorator, self).addSuccess(test, details)

    def addError(self, test, err=None, details=None):
        self._log(logging.ERROR, "ERROR: %s" % (test.id()))
        if hasattr(test, "getDetails"):
            self._log_details(logging.ERROR, test.getDetails())
        return super(type(self), self).addError(test, err, details)

    def addFailure(self, test, err=None, details=None):
        """Called for a test which failed an assert"""
        self._log(logging.ERROR, "FAIL: %s" % (test.id()))
        if hasattr(test, "getDetails"):
            self._log_details(logging.ERROR, test.getDetails())
        return super(type(self), self).addFailure(test, err, details)


def get_output_formats():
    """Get information regarding the different output formats supported."""
    supported_formats = {}
    supported_formats['status'] = _construct_status
    supported_formats['text'] = _construct_text
    if try_import('junitxml'):
        supported_formats['xml'] = _construct_xml
    if try_import('subunit'):
        supported_formats['subunit'] = _construct_subunit
    return supported_formats


def get_default_format():
    return 'status'


def _construct_xml(**kwargs):
    from junitxml import JUnitXmlResult
    stream = kwargs.pop('stream')
    failfast = kwargs.pop('failfast')
    _raise_on_unknown_kwargs(kwargs)
    result = ExtendedToOriginalDecorator(JUnitXmlResult(stream))
    result.failfast = failfast
    return result


def _construct_status(**kwargs):
    """Status (terse) text output."""
    stream = kwargs.pop('stream')
    failfast = kwargs.pop('failfast')
    _raise_on_unknown_kwargs(kwargs)
    return StatusFormatTestResult(stream, failfast)


def _construct_text(**kwargs):
    """Verbose text output."""
    stream = kwargs.pop('stream')
    failfast = kwargs.pop('failfast')
    _raise_on_unknown_kwargs(kwargs)
    return LoggedTestResultDecorator(TextTestResult(stream, failfast))


def _construct_subunit(**kwargs):
    from subunit import StreamResultToBytes
    stream = kwargs.pop('stream')
    failfast = kwargs.pop('failfast')
    _raise_on_unknown_kwargs(kwargs)
    result = ExtendedToStreamDecorator(StreamResultToBytes(stream))
    result.failfast = failfast
    return result
